@@ -137,7 +137,7 @@ KNumber CalcEngine_p::moveIntoGradInterval(const KNumber &num)
 
 KNumber CalcEngine_p::ArcCosDeg(const KNumber &input)
 {
-    if (input.type() == KNumber::TypeError || input < -KNumber::One || input > KNumber::One) {
+    if (input.type() == KNumber::TypeError) {
         return KNumber::NaN;
     }
 
@@ -157,7 +157,7 @@ KNumber CalcEngine_p::ArcCosDeg(const KNumber &input)
 
 KNumber CalcEngine_p::ArcCosRad(const KNumber &input)
 {
-    if (input.type() == KNumber::TypeError || input < -KNumber::One || input > KNumber::One) {
+    if (input.type() == KNumber::TypeError) {
         return KNumber::NaN;
     }
     return input.acos();
@@ -165,7 +165,7 @@ KNumber CalcEngine_p::ArcCosRad(const KNumber &input)
 
 KNumber CalcEngine_p::ArcCosGrad(const KNumber &input)
 {
-    if (input.type() == KNumber::TypeError || input < -KNumber::One || input > KNumber::One) {
+    if (input.type() == KNumber::TypeError) {
         return KNumber::NaN;
     }
     if (input.type() == KNumber::TypeInteger) {
@@ -184,7 +184,7 @@ KNumber CalcEngine_p::ArcCosGrad(const KNumber &input)
 
 KNumber CalcEngine_p::ArcSinDeg(const KNumber &input)
 {
-    if (input.type() == KNumber::TypeError || input < -KNumber::One || input > KNumber::One) {
+    if (input.type() == KNumber::TypeError) {
         return KNumber::NaN;
     }
     if (input.type() == KNumber::TypeInteger) {
@@ -203,7 +203,7 @@ KNumber CalcEngine_p::ArcSinDeg(const KNumber &input)
 
 KNumber CalcEngine_p::ArcSinRad(const KNumber &input)
 {
-    if (input.type() == KNumber::TypeError || input < -KNumber::One || input > KNumber::One) {
+    if (input.type() == KNumber::TypeError) {
         return KNumber::NaN;
     }
     return input.asin();
@@ -211,7 +211,7 @@ KNumber CalcEngine_p::ArcSinRad(const KNumber &input)
 
 KNumber CalcEngine_p::ArcSinGrad(const KNumber &input)
 {
-    if (input.type() == KNumber::TypeError || input < -KNumber::One || input > KNumber::One) {
+    if (input.type() == KNumber::TypeError) {
         return KNumber::NaN;
     }
     if (input.type() == KNumber::TypeInteger) {
@@ -281,9 +281,6 @@ KNumber CalcEngine_p::AreaCosHyp(const KNumber &input)
             return KNumber::NaN;
     }
 
-    if (input < KNumber::One) {
-        return KNumber::NaN;
-    }
     if (input == KNumber::One) {
         return KNumber::Zero;
     }
@@ -313,9 +310,6 @@ KNumber CalcEngine_p::AreaTangensHyp(const KNumber &input)
         return KNumber::NaN;
     }
 
-    if (input < -KNumber::One || input > KNumber::One) {
-        return KNumber::NaN;
-    }
     if (input == KNumber::One) {
         return KNumber::PosInfinity;
     }
@@ -479,11 +473,7 @@ KNumber CalcEngine_p::InvertSign(const KNumber &input)
 
 KNumber CalcEngine_p::Ln(const KNumber &input)
 {
-    if (input < KNumber::Zero)
-        return KNumber::NaN;
-    else if (input == KNumber::Zero)
-        return KNumber::NegInfinity;
-    else if (input == KNumber::One)
+    if (input == KNumber::One)
         return KNumber::Zero;
     else {
         return input.ln();
@@ -492,11 +482,7 @@ KNumber CalcEngine_p::Ln(const KNumber &input)
 
 KNumber CalcEngine_p::Log10(const KNumber &input)
 {
-    if (input < KNumber::Zero)
-        return KNumber::NaN;
-    else if (input == KNumber::Zero)
-        return KNumber::NegInfinity;
-    else if (input == KNumber::One)
+    if (input == KNumber::One)
         return KNumber::Zero;
     else {
         return input.log10();
@@ -697,6 +683,42 @@ KNumber CalcEngine_p::TangensGrad(const KNumber &input)
     return arg1 / arg2;
 }
 
+KNumber CalcEngine_p::PolarDeg(const KNumber &input)
+{
+    if (input.type() == KNumber::TypeError) {
+        return KNumber::NaN;
+    }
+
+    KNumber arg1 = CosDeg(input);
+    KNumber arg2 = SinDeg(input);
+
+    return arg1 + arg2 * KNumber(QStringLiteral("i"));
+}
+
+KNumber CalcEngine_p::PolarRad(const KNumber &input)
+{
+    if (input.type() == KNumber::TypeError) {
+        return KNumber::NaN;
+    }
+
+    KNumber arg1 = CosRad(input);
+    KNumber arg2 = SinRad(input);
+
+    return arg1 + arg2 * KNumber(QStringLiteral("i"));
+}
+
+KNumber CalcEngine_p::PolarGrad(const KNumber &input)
+{
+    if (input.type() == KNumber::TypeError) {
+        return KNumber::NaN;
+    }
+
+    KNumber arg1 = CosGrad(input);
+    KNumber arg2 = SinGrad(input);
+
+    return arg1 + arg2 * KNumber(QStringLiteral("i"));
+}
+
 KNumber CalcEngine_p::TangensHyp(const KNumber &input)
 {
     if (input.type() == KNumber::TypeError) {
@@ -711,3 +733,51 @@ KNumber CalcEngine_p::TangensHyp(const KNumber &input)
 
     return input.tanh();
 }
+
+KNumber CalcEngine_p::Conjugate(const KNumber &input)
+{
+    return input.conj();
+}
+
+KNumber CalcEngine_p::ArgumentDeg(const KNumber &input)
+{
+    if (input.realPart() == KNumber::Zero) {
+        return (input.imaginaryPart() > KNumber::Zero) ? KNumber(90) : KNumber(-90);
+    } else if (input.imaginaryPart() == KNumber::Zero) {
+        return (input.realPart() >= KNumber::Zero) ? KNumber::Zero : KNumber(180);
+    } else {
+        return Rad2Deg(input.arg());
+    }
+}
+
+KNumber CalcEngine_p::ArgumentGrad(const KNumber &input)
+{
+    if (input.realPart() == KNumber::Zero) {
+        return (input.imaginaryPart() > KNumber::Zero) ? KNumber(100) : KNumber(-100);
+    } else if (input.imaginaryPart() == KNumber::Zero) {
+        return (input.realPart() >= KNumber::Zero) ? KNumber::Zero : KNumber(200);
+    } else {
+        return Rad2Gra(input.arg());
+    }
+}
+
+KNumber CalcEngine_p::ArgumentRad(const KNumber &input)
+{
+    if (input.realPart() == KNumber::Zero) {
+        return (input.imaginaryPart() > KNumber::Zero) ? KNumber::Pi() / KNumber(2) : -KNumber::Pi() / KNumber(2);
+    } else if (input.imaginaryPart() == KNumber::Zero) {
+        return (input.realPart() >= KNumber::Zero) ? KNumber::Zero : KNumber::Pi();
+    } else {
+        return input.arg();
+    }
+}
+
+KNumber CalcEngine_p::Real(const KNumber &input)
+{
+    return input.realPart();
+}
+
+KNumber CalcEngine_p::Imaginary(const KNumber &input)
+{
+    return input.imaginaryPart();
+}
