@@ -5,8 +5,8 @@
 
 #include "kcalc_input_display.h"
 
+#include <QKeyEvent>
 #include <QLineEdit>
-// #include <kcalc_parser.h>
 
 //------------------------------------------------------------------------------
 // Name: KCalcInputDisplay
@@ -17,7 +17,7 @@ KCalcInputDisplay::KCalcInputDisplay(QWidget *parent)
 {
     overwrite_ = false;
     hard_overwrite_ = false;
-    function_wrap_ = false;
+    has_result_ = false;
 }
 
 //------------------------------------------------------------------------------
@@ -30,12 +30,15 @@ KCalcInputDisplay::~KCalcInputDisplay() = default;
 // Name: reset
 // Desc:
 //------------------------------------------------------------------------------
-void KCalcInputDisplay::reset()
+void KCalcInputDisplay::reset(bool clearRedoUndo /*=false*/)
 {
     this->clear();
     overwrite_ = false;
     hard_overwrite_ = false;
-    function_wrap_ = false;
+    has_result_ = false;
+    if (clearRedoUndo) {
+        this->setText(QStringLiteral(""));
+    }
 }
 
 //------------------------------------------------------------------------------
@@ -51,7 +54,23 @@ void KCalcInputDisplay::insertToken(const QString &token)
     }
 
     this->insert(token);
-    function_wrap_ = false;
+    has_result_ = false;
+}
+
+//------------------------------------------------------------------------------
+// Name: insertTokenNumeric
+// Desc:
+//------------------------------------------------------------------------------
+void KCalcInputDisplay::insertTokenNumeric(const QString &token)
+{
+    if (overwrite_ || hard_overwrite_ || has_result_) {
+        this->clear();
+        overwrite_ = false;
+        hard_overwrite_ = false;
+        has_result_ = false;
+    }
+
+    this->insert(token);
 }
 
 //------------------------------------------------------------------------------
@@ -66,7 +85,7 @@ void KCalcInputDisplay::insertTokenFunction(const QString &token)
         this->clear();
         this->insert(token);
         this->insert(QStringLiteral("("));
-    } else if (function_wrap_) {
+    } else if (has_result_) {
         this->home(false);
         this->insert(token);
         this->insert(QStringLiteral("("));
@@ -79,12 +98,13 @@ void KCalcInputDisplay::insertTokenFunction(const QString &token)
 }
 
 //------------------------------------------------------------------------------
-// Name: slotSetFunctionWrap
+// Name: setHasResult
 // Desc:
 //------------------------------------------------------------------------------
-void KCalcInputDisplay::slotSetFunctionWrap()
+void KCalcInputDisplay::setHasResult()
 {
-    function_wrap_ = true;
+    has_result_ = true;
+    this->clearFocus();
 }
 
 //------------------------------------------------------------------------------
@@ -116,12 +136,12 @@ void KCalcInputDisplay::slotSetHardOverwrite()
 }
 
 //------------------------------------------------------------------------------
-// Name: slotClearFunctionWrap
+// Name: clearHasResult
 // Desc:
 //------------------------------------------------------------------------------
-void KCalcInputDisplay::slotClearFunctionWrap()
+void KCalcInputDisplay::clearHasResult()
 {
-    function_wrap_ = false;
+    has_result_ = false;
 }
 
 //------------------------------------------------------------------------------
@@ -145,6 +165,47 @@ void KCalcInputDisplay::slotClearOverwrite()
     }
 }
 
+//------------------------------------------------------------------------------
+// Name: keyPressEvent
+// Desc:
+//------------------------------------------------------------------------------
+void KCalcInputDisplay::keyPressEvent(QKeyEvent *event)
+{
+    if (event->matches(QKeySequence::Undo)) {
+        undoCalculation();
+    } else if (event->matches(QKeySequence::Redo)) {
+        redoCalculation();
+    } else {
+        QLineEdit::keyPressEvent(event);
+    }
+}
+
+//------------------------------------------------------------------------------
+// Name: undoCalculation
+// Desc:
+//------------------------------------------------------------------------------
+void KCalcInputDisplay::undoCalculation()
+{
+    QLineEdit::undo();
+    if (this->isUndoAvailable() && text().isEmpty()) {
+        QLineEdit::undo();
+        QLineEdit::deselect();
+    }
+}
+
+//------------------------------------------------------------------------------
+// Name: redoCalculation
+// Desc:
+//------------------------------------------------------------------------------
+void KCalcInputDisplay::redoCalculation()
+{
+    QLineEdit::redo();
+    if (this->isRedoAvailable() && text().isEmpty()) {
+        QLineEdit::redo();
+        QLineEdit::deselect();
+    }
+}
+
 //------------------------------------------------------------------------------
 // Name: focusInEvent
 // Desc: focusIn, if hardOverwrite, display is cleared,
@@ -154,7 +215,7 @@ void KCalcInputDisplay::focusInEvent(QFocusEvent *e)
 {
     QLineEdit::focusInEvent(e);
     slotClearOverwrite();
-    slotClearFunctionWrap();
+    clearHasResult();
 }
 
 #include "moc_kcalc_input_display.cpp"
