@@ -20,6 +20,7 @@ class KNumberFraction : public KNumberBase
     friend class KNumberError;
     friend class KNumberInteger;
     friend class KNumberFloat;
+    friend class KNumberComplex;
 
 public:
     static bool defaultFractionalInput;
@@ -47,6 +48,7 @@ public:
     qint64 toInt64() const override;
 
 public:
+    bool isReal() const override;
     bool isInteger() const override;
     bool isZero() const override;
     int sign() const override;
@@ -84,6 +86,10 @@ public:
     KNumberBase *ceil() override;
     KNumberBase *exp10() override;
     KNumberBase *exp() override;
+    KNumberBase *realPart() override;
+    KNumberBase *imaginaryPart() override;
+    KNumberBase *arg() override;
+    KNumberBase *conj() override;
     KNumberBase *bin(KNumberBase *rhs) override;
 
 public:
@@ -114,7 +120,9 @@ private:
 #if 0
 	// TODO: this is omitted because there is no good way to
 	// implement it
-	KNumberFraction(const knumber_float *value);
+    // truncate?
+	KNumberFraction(const KNumberFloat *value);
+    KNumberFraction(const KNumberComplex *value);
 #endif
     explicit KNumberFraction(const KNumberError *value);
 
